@@ -12,17 +12,17 @@
 from __future__ import annotations
 
 import json
+import re
 from functools import cached_property
 from typing import ClassVar, Dict, List, Tuple
 
 from autotransform.batcher.base import Batch
 from autotransform.change.base import Change, ChangeName, ChangeState, ReviewState, TestState
-from autotransform.config import get_repo_config_relative_path
+from autotransform.config import get_config
 from autotransform.item.base import FACTORY as item_factory
-from autotransform.schema.builder import FACTORY as schema_builder_factory
 from autotransform.schema.schema import AutoTransformSchema
-from autotransform.util.enums import SchemaType
 from autotransform.util.github import GithubUtils, PullRequest
+from autotransform.util.schema_map import SchemaMap
 
 
 class GithubChange(Change):
@@ -46,7 +46,7 @@ class GithubChange(Change):
             Batch: The Batch used to produce the Change.
         """
 
-        return self._body_data[1]
+        return self._automation_data[1]
 
     def get_schema(self) -> AutoTransformSchema:
         """Gets the Schema that was used to produce the Change.
@@ -56,17 +56,11 @@ class GithubChange(Change):
         """
 
         schema_name = self.get_schema_name()
-        map_file_path = f"{get_repo_config_relative_path()}/schema_map.json"
-        with open(map_file_path, "r", encoding="UTF-8") as map_file:
-            schema_map = json.loads(map_file.read())
-        data = schema_map[schema_name]
-        schema_type = SchemaType(data["type"])
-        if schema_type == SchemaType.BUILDER:
-            schema = schema_builder_factory.get_instance({"name": data["target"]}).build()
-        else:
-            with open(data["target"], "r", encoding="utf-8") as schema_file:
-                schema = AutoTransformSchema.from_data(json.loads(schema_file.read()))
+        schema = SchemaMap.get().get_schema(schema_name)
         assert schema_name == schema.config.schema_name
+        repo_override = get_config().repo_override
+        if repo_override is not None:
+            schema.repo = repo_override
 
         return schema
 
@@ -77,7 +71,7 @@ class GithubChange(Change):
             str: The name of the Schema.
         """
 
-        return self._pull_request.branch.split("/")[1].replace("_", " ")
+        return self._automation_data[0].config.schema_name
 
     def get_state(self) -> ChangeState:
         """Gets the current state of the Change.
@@ -280,7 +274,7 @@ class GithubChange(Change):
         return GithubUtils.get(self.full_github_name).get_pull_request(self.pull_number)
 
     @cached_property
-    def _body_data(self) -> Tuple[AutoTransformSchema, Batch]:
+    def _automation_data(self) -> Tuple[AutoTransformSchema, Batch]:
         """Loads the Schema and Batch data for the GithubChange as a cached property.
 
         Returns:
@@ -288,22 +282,31 @@ class GithubChange(Change):
                 in the PullRequest's Body.
         """
 
-        data: Dict[str, List[str]] = {"schema": [], "batch": []}
-        cur_line_placement = None
-        for line in self._pull_request.body.splitlines():
-            if line == GithubUtils.BEGIN_SCHEMA:
-                cur_line_placement = "schema"
-            elif line == GithubUtils.END_SCHEMA:
-                cur_line_placement = None
-            elif line == GithubUtils.BEGIN_BATCH:
-                cur_line_placement = "batch"
-            elif line == GithubUtils.END_BATCH:
-                cur_line_placement = None
-            elif cur_line_placement is not None:
-                data[cur_line_placement].append(line)
+        data: Dict[str, str] = {}
+        gist_match = re.search("<<<Automation Info Gist: (.*)>>>", self._pull_request.body)
+        if gist_match:
+            gist = GithubUtils.get(self.full_github_name).get_gist(gist_match.groups()[0])
+            data["schema"] = gist.get_file_content("schema") or ""
+            data["batch"] = gist.get_file_content("batch") or ""
+        else:
+            cur_line_placement = None
+            data_lines: Dict[str, List[str]] = {"schema": [], "batch": []}
+            for line in self._pull_request.body.splitlines():
+                if line == GithubUtils.BEGIN_SCHEMA:
+                    cur_line_placement = "schema"
+                elif line == GithubUtils.END_SCHEMA:
+                    cur_line_placement = None
+                elif line == GithubUtils.BEGIN_BATCH:
+                    cur_line_placement = "batch"
+                elif line == GithubUtils.END_BATCH:
+                    cur_line_placement = None
+                elif cur_line_placement is not None:
+                    data_lines[cur_line_placement].append(line)
+            data["schema"] = "\n".join(data_lines["schema"])
+            data["batch"] = "\n".join(data_lines["batch"])
 
-        schema = AutoTransformSchema.from_data(json.loads("\n".join(data["schema"])))
-        batch = json.loads("\n".join(data["batch"]))
+        schema = AutoTransformSchema.from_data(json.loads(data["schema"]))
+        batch = json.loads(data["batch"])
         items = [item_factory.get_instance(item) for item in batch["items"]]
         batch = {
             "items": items,
