@@ -14,16 +14,17 @@ from __future__ import annotations
 import time
 from datetime import datetime, timedelta
 from functools import cached_property
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple
 from urllib.error import HTTPError
 
 import pytz
-from fastcore.basics import AttrDict  # type: ignore
-from ghapi.all import GhApi, gh2date  # type: ignore
-
 from autotransform.config import get_config
 from autotransform.event.debug import DebugEvent
 from autotransform.event.handler import EventHandler
+from autotransform.event.warning import WarningEvent
+from autotransform.repo.git import GitRepo
+from fastcore.basics import AttrDict  # type: ignore
+from ghapi.all import GhApi, gh2date  # type: ignore
 
 
 class GithubUtils:
@@ -33,11 +34,17 @@ class GithubUtils:
     Attributes:
         __instances (Dict[str, GithubUtils]): A mapping of repo names to Util objects.
         _api (GhApi): The API object used to handle requests to Github.
+        _fully_qualified_repo (str): The fully qualified name of the Github repo.
+        _gists (Dict[str, Gist]): A mapping from id to Gist.
+        _pulls (Dict[int, PullRequest]): A mapping from pull number to PullRequest.
     """
 
     __instances: Dict[str, GithubUtils] = {}
 
     _api: GhApi
+    _fully_qualified_repo: str
+    _gists: Dict[str, Gist]
+    _pulls: Dict[int, PullRequest]
 
     BEGIN_SCHEMA: str = "<<<<BEGIN SCHEMA>>>>"
     END_SCHEMA: str = "<<<<END SCHEMA>>>>"
@@ -58,6 +65,9 @@ class GithubUtils:
         url = get_config().github_base_url
         repo_parts = fully_qualified_repo.split("/")
         self._api = GhApi(token=token, gh_host=url, owner=repo_parts[0], repo=repo_parts[1])
+        self._fully_qualified_repo = fully_qualified_repo
+        self._gists: Dict[str, Gist] = {}
+        self._pulls: Dict[int, PullRequest] = {}
 
     @staticmethod
     def get(fully_qualified_repo: str) -> GithubUtils:
@@ -98,10 +108,10 @@ class GithubUtils:
         """
 
         pull = self._api.pulls.create(title=title, head=head, base=base, body=body)
-        return PullRequest(self._api, pull)
+        return PullRequest(self._api, pull.number)
 
     def get_pull_request(self, pull_number: int) -> PullRequest:
-        """Gets a pull request from the Github API.
+        """Gets a pull request.
 
         Args:
             pull_number (int): The number of the pull request.
@@ -110,7 +120,40 @@ class GithubUtils:
             PullRequest: The pull request.
         """
 
-        return PullRequest(self._api, self._api.pulls.get(pull_number=pull_number))
+        if pull_number not in self._pulls:
+            self._pulls[pull_number] = PullRequest(self._api, pull_number)
+        return self._pulls[pull_number]
+
+    def create_gist(
+        self, files: Dict[str, Dict[str, str]], description: str, public: bool = True
+    ) -> Gist:
+        """Creates a Gist containing the supplied information
+
+        Args:
+            files (Dict[str, Dict[str, str]]): The files to include in the gist.
+            description (str): A simple description of the gist.
+            public (bool, optional): Whether the gist should be public. Defaults to True.
+
+        Returns:
+            Gist: The created gist.
+        """
+
+        res = self._api.gists.create(description=description, files=files, public=public)
+        return Gist(self._api, res.id)
+
+    def get_gist(self, gist_id: str) -> Gist:
+        """Gets a wrapper around the requested gist.
+
+        Args:
+            gist_id (str): The id of the gist.
+
+        Returns:
+            Gist: The requested gist.
+        """
+
+        if gist_id not in self._gists:
+            self._gists[gist_id] = Gist(self._api, gist_id)
+        return self._gists[gist_id]
 
     def get_open_pull_requests(self, base: Optional[str] = None) -> List[PullRequest]:
         """Gets all outstanding pull requests from the repo.
@@ -123,8 +166,25 @@ class GithubUtils:
             List[PullRequest]: The list of all requests outstanding for the repo.
         """
 
-        pulls = self._api.pulls.list(state="open", base=base, sort="created", direction="desc")
-        return [PullRequest(self._api, pull) for pull in pulls]
+        username = self._api.users.get_authenticated().login
+        query = f"type:pr state:open author:{username} repo:{self._fully_qualified_repo}"
+        if base is not None:
+            query = f"{query} base:{base}"
+        page = 1
+        num_per_page = 100
+        fetch_more = True
+        all_pulls: Set[int] = set()
+        while fetch_more:
+            prs = self._api.search.issues_and_pull_requests(
+                q=query, sort="created", order="desc", per_page=num_per_page, page=page
+            )
+            fetch_more = len(prs["items"]) == num_per_page
+            page = page + 1
+            all_pulls = all_pulls.union([pr.number for pr in prs["items"]])
+
+        pulls = [self.get_pull_request(pull_number) for pull_number in all_pulls]
+
+        return [pull for pull in pulls if pull.branch.startswith(GitRepo.BRANCH_NAME_PREFIX)]
 
     def create_workflow_dispatch(
         self, workflow: str | int, ref: str, inputs: Dict[str, Any]
@@ -160,59 +220,67 @@ class GithubUtils:
             return res.workflow_runs[0].html_url
         except HTTPError as err:
             EventHandler.get().handle(
-                DebugEvent({"message": f"Failed dispatch workflow {workflow}: {err}"})
+                WarningEvent({"message": f"Failed dispatch workflow {workflow}: {err}"})
             )
             return None
 
 
+# pylint: disable=too-many-public-methods
 class PullRequest:
     """A wrapper around GhApi pull request response for simplified access to pull request info.
 
     Attributes:
-        body (str): The body of the pull request.
-        branch (str): The branch of the pull request.
-        merged (bool): Whether the pull request has been merged.
         number (int): The number of the pull request.
         _api (GhApi): The API object used to access Github's API.
-        _state (str): The state of the pull request.
     """
 
-    # pylint: disable=too-many-instance-attributes
-
-    body: str
-    branch: str
-    merged: bool
     number: int
-    owner_id: int
 
     _api: GhApi
-    _state: str
 
-    def __init__(self, api: GhApi, pull: AttrDict):
-        self.body = pull.body
-        self.branch = pull.head.ref
-        self.merged = pull.merged if "merged" in pull else False
-        self.number = pull.number
-        self.owner_id = pull.user.id
-        self._created_at = pull.created_at
-        self._updated_at = pull.updated_at
+    def __init__(self, api: GhApi, pull_number: int):
+        self.number = pull_number
         self._api = api
-        self._state = pull.state
 
-    @staticmethod
-    def get_from_number(api: GhApi, pull_number: int) -> PullRequest:
-        """Gets a pull request with the given number.
-
-        Args:
-            api (GhApi): The API object used to access Github's API.
-            pull_number (int): The number of the pull request.
+    @property
+    def body(self) -> str:
+        """Gets the body of the pull request.
 
         Returns:
-            PullRequest: The pull request associated with the number.
+            str: The body of the pull request.
         """
 
-        pull = api.pulls.get(pull_number=pull_number)
-        return PullRequest(api, pull)
+        return self._detailed_info.body
+
+    @property
+    def branch(self) -> str:
+        """Gets the name of the head branch for the pull request.
+
+        Returns:
+            str: The name of the head branch.
+        """
+
+        return self._detailed_info.head.ref
+
+    @property
+    def merged(self) -> bool:
+        """Whether the pull request has been merged.
+
+        Returns:
+            bool: Whether the pull request was merged.
+        """
+
+        return self._detailed_info.merged
+
+    @property
+    def owner_id(self) -> int:
+        """The ID of the user that created the pull request.
+
+        Returns:
+            int: The ID of the owner of the pull request.
+        """
+
+        return self._detailed_info.user.id
 
     def is_open(self) -> bool:
         """A simple check if the pull request is open.
@@ -221,7 +289,7 @@ class PullRequest:
             bool: Whether the pull request is open.
         """
 
-        return self._state == "open"
+        return self._detailed_info.state == "open"
 
     def is_closed(self) -> bool:
         """A simple check if the pull request is closed.
@@ -303,6 +371,16 @@ class PullRequest:
 
         return self._reviewers[1]
 
+    @cached_property
+    def _detailed_info(self) -> AttrDict:
+        """A cached value of the detailed info from the Github API.
+
+        Returns:
+            AttrDict: The detailed info of the PR.
+        """
+
+        return self._api.pulls.get(pull_number=self.number)
+
     @cached_property
     def _reviewers(self) -> Tuple[List[str], List[str]]:
         """A cached value of the reviewers of the change.
@@ -311,11 +389,19 @@ class PullRequest:
             Tuple[List[str], List[str]]: A tuple containing user reviewers and team reviewers.
         """
 
-        reviewers = self._api.pulls.list_requested_reviewers(pull_number=self.number)
-        return (
-            [reviewer["login"] for reviewer in reviewers["users"]],
-            [reviewer["name"] for reviewer in reviewers["teams"]],
-        )
+        requested_reviewers = self._api.pulls.list_requested_reviewers(pull_number=self.number)
+        per_page = 50
+        page = 1
+        user_reviewers = [reviewer["login"] for reviewer in requested_reviewers["users"]]
+        team_reviewers = [reviewer["name"] for reviewer in requested_reviewers["teams"]]
+        more_reviewers = True
+        while more_reviewers:
+            reviews = self._api.pulls.list_reviews(
+                pull_number=self.number, page=page, per_page=per_page
+            )
+            user_reviewers.extend(review["user"]["login"] for review in reviews)
+            more_reviewers = len(reviews) == per_page
+        return (list(set(user_reviewers)), list(set(team_reviewers)))
 
     def get_created_at(self) -> int:
         """Gets the created at timestamp.
@@ -323,7 +409,7 @@ class PullRequest:
         Returns:
             int: The timestamp for when the pull request was created.
         """
-        return int(pytz.utc.localize(gh2date(self._created_at)).timestamp())
+        return int(pytz.utc.localize(gh2date(self._detailed_info.created_at)).timestamp())
 
     def get_updated_at(self) -> int:
         """Gets the updated at timestamp.
@@ -331,7 +417,19 @@ class PullRequest:
         Returns:
             int: The timestamp for when the pull request was updated.
         """
-        return int(pytz.utc.localize(gh2date(self._updated_at)).timestamp())
+        return int(pytz.utc.localize(gh2date(self._detailed_info.updated_at)).timestamp())
+
+    def get_mergeable_state(self) -> str:
+        """Gets the mergeable state of the PR.
+
+        Returns:
+            str: The mergeable state.
+        """
+        return (
+            str(self._detailed_info.mergeable_state)
+            if "mergeable_state" in self._detailed_info
+            else "unknown"
+        )
 
     def add_labels(self, labels: List[str]) -> None:
         """Adds a list of labels to the pull request.
@@ -377,7 +475,7 @@ class PullRequest:
             return res.merged
         except HTTPError as err:
             EventHandler.get().handle(
-                DebugEvent({"message": f"Failed merge pull request {self.number}: {err}"})
+                WarningEvent({"message": f"Failed merge pull request {self.number}: {err}"})
             )
             return False
 
@@ -402,7 +500,7 @@ class PullRequest:
             return res.state == "closed"
         except HTTPError as err:
             EventHandler.get().handle(
-                DebugEvent({"message": f"Failed to close pull request {self.number}: {err}"})
+                WarningEvent({"message": f"Failed to close pull request {self.number}: {err}"})
             )
             return False
 
@@ -418,6 +516,57 @@ class PullRequest:
             return True
         except HTTPError as err:
             EventHandler.get().handle(
-                DebugEvent({"message": f"Failed to delete branch {self.branch}: {err}"})
+                WarningEvent({"message": f"Failed to delete branch {self.branch}: {err}"})
             )
             return False
+
+
+class Gist:
+    """A wrapper around GhApi gist response for simplified access to gist info.
+
+    Attributes:
+        id (str): The id of the gist.
+        _api (GhApi): The API object used to access Github's API.
+    """
+
+    gist_id: str
+
+    _api: GhApi
+
+    def __init__(self, api: GhApi, gist_id: str):
+        self.gist_id = gist_id
+        self._api = api
+
+    @cached_property
+    def _detailed_info(self) -> AttrDict:
+        """A cached value of the detailed info from the Github API.
+
+        Returns:
+            AttrDict: The detailed info of the gist.
+        """
+
+        return self._api.gists.get(gist_id=self.gist_id)
+
+    def get_description(self) -> str:
+        """Gets the description of the gist.
+
+        Returns:
+            str: The description of the gist.
+        """
+
+        return self._detailed_info.description
+
+    def get_file_content(self, file_name: str) -> Optional[str]:
+        """Gets the content of the requested file within the gist.
+
+        Args:
+            file_name (str): The name of the file to get the content for.
+
+        Returns:
+            Optional[str]: The contents of the file. None if the gist does not have a file with
+                the supplied name.
+        """
+
+        if file_name not in self._detailed_info.files:
+            return None
+        return self._detailed_info.files[file_name].content
